c----------------------------------------------------------------
c --- PRELND1 -- SURFACE CHARACTERISTICS PREPREPROCESSOR
c----------------------------------------------------------------
c
c --- PRELND1 Version: 3.0           Level: 961113           MAIN
c
C     THIS PROGRAM TAKES THE GIS LAND COVER DATA BASE
C     WHICH COVERS THE ENTIRE USA AND MAPS IN INTO
C     THE CALMET MODELING DOMAIN.  The output data file is ready
c...  for input to LU2CAL.FOR
c
c...  Note:  The data file, 'GIS.DAT', has (Long./Lat.) resolution of
c...         (1/4, 1/6),  roughly (28, 37.1) in kilometers.
c
c...  Program originally in SAI's ARM3 package.  3/88
c
c...  Modified by Ed Chang, SRC; 8/93 for CALMET use A314 (CAPTEX)
c....   (1) Add features to do domains projected by Lambert conformal
c...        conic projection (LC) (instead of in UTM zones).
c...        IMAP = 1:   Lambert Conformal conic-projected Cartesian
c...                    coordinates
c...        IMAP = 0:   Cartesian coordinates in UTM zone
c
c...    (2) For A314 application, domain resolution was 18 Km, smaller
c...        than the GIS.DAT data resolution.  Therefore, original
c...        call to subroutine INSIDE & SMOOTH was eliminated.
C
c
c                 EARTH TECH/Sigma Research Corporation
c                 196 Baker Avenue
c                 Concord, MA 01742
c                 (508) 371-4200
c                 (508) 371-2468 (fax)
c
c --------------------------------------------------------------
C     PROGRAM PRELND
C
      PARAMETER (MXX=104)
      PARAMETER (MXY=90)
      COMMON /DATA2/ PLAND(11,MXX,MXY)
      COMMON /DATA4/ UTMX1,UTMY1,UTMX2,UTMY2,UTMX3,UTMY3,UTMX4,UTMY4
      DIMENSION IPERC(11)
      DIMENSION TOT(11)
      LOGICAL lutm
      CHARACTER*80 FN
      character*25 title
      character*20 namlnd(11)
      dimension percck(mxx,mxy),replu(mxx,mxy)
      dimension dum1d(mxx*mxy)
C
C     GIS DATA BASE HAS
C     11 LAND USE CLASSES:   URBAN
C                            AGRICULTURE
C                            RANGELAND
C                            DECIDUOUS FOREST
C                            CONFIEROUS FOREST
C                            MIXED FOREST
C                            WATER
C                            BARREN LAND
C                            NON-FORESTED WETLAND
C                            MIXED AG-RANGELAND
C                            ROCKY OPEN AREAS WITH LOW SHRUBS AND LICHENS
C
C     SURFACE ROUGHNESS VALUES (CM) BY GIS LAND USE TYPOE
c
      DATA NAMLND/'URBAN               ',
     1            'AGRICULTURE         ',
     2            'RANGELAND           ',
     3            'DECIDUOUS FOREST    ',
     4            'CONIFEROUS FOREST   ',
     5            'MIXED FOREST        ',
     6            'WATER               ',
     7            'BARREN LAND         ',
     8            'NON-FORESTED WETLAND',
     9            'MIXED AG-RANGELAND  ',
     9            'ROCKY OR LOW SHRUBS '/
C
c     DATA Z00/100.0,25.0,3.0,100.0,100.0,100.0,0.02,0.1,0.5,25.0,0.5/
C
C     OPEN FILES
C
      OPEN(UNIT=11,FILE='prelnd.in',FORM='FORMATTED')
      READ(11,1111) FN
      OPEN(UNIT=7,FILE=FN,FORM='FORMATTED')
      print *,'"GIS.DAT" file opened at unit 7: ',fn
      READ(11,1111) FN
      OPEN(UNIT=12,FILE=FN,FORM='FORMATTED')
      print *,'Output list file opened at unit 12: ',fn
      READ(11,1111) FN
      OPEN(UNIT=8,FILE=FN,FORM='FORMATTED')
      print *,'Output ASCII data file for plotting opened at unit 8: ',
     *        fn
1111  FORMAT(A80)
C
      DLAT=1./12.+1.E-6
      DLONG=1./8.+1.E-6
C
C     FIRST CONVERT UNITS FROM CM TO M
C
c     DO 10 J=1,11
c     Z00(J)=Z00(J)*0.01
c10    CONTINUE
c
c...  Initialization
      do 101 i=1,mxx
      do 101 j=1,mxy
        percck(i,j)=0.0
        replu(i,j)=0.0
      do 101 k=1,11
        pland(k,i,j)=0.0
101   continue
C
C     READ IN GRID DEFINITION
C
      READ(11,1002) XO,YO,ICS,NXC,NYC
      WRITE(12,3002) XO,YO,ICS,NXC,NYC
      read(11,3003) imap
3003  format(8i10)
      if(imap .eq. 1)then
          read(11,3002) rlon,rlat
          lutm=.false.
      elseif (imap .eq. 0)then
          read(11,3003) izone
          lutm=.true.
      else
          print *,'Error: IMAP should be either 1 or 2: ',imap
          stop 'imap'
      endif
3002  FORMAT(2F10.3,4I10)
1002  FORMAT(2F10.0,4I10)
      IXO=NINT(XO)
      IYO=NINT(YO)
      IBSZN=IZONE
C
C     WRITE OUT SUMMARY INFORMATION
C
      WRITE(12,2101) IBSZN,IXO,IBSZN,IYO,NXC,NYC,ICS
2101  FORMAT(1H1,//,10X,'***SUMMARY GRID INFORMATION FOR PROGRAM PRELND'
     1,'-LANDUSE PREPROCESSOR ',/,20X,'UTM ZONE ',I2,' X ORIGIN=',I5,' (
     2KM)',/,20X,'UTM ZONE ',I2,' Y ORIGIN=',I5,' (KM)',/,20X
     2,'NUMBER OF X GRIDS =',I5
     3,/,20X,'NUMBER OF Y GRIDS =',I5,/,20X,'GRID CELL SPACING =',I4
     4,' (KM)')
C
C     DETERMINE WINDOW FOR LOOKING AT GIS DATA
C
      UTMX=FLOAT(IXO)
      UTMY=FLOAT(IYO)
      if(lutm)then
          call glat(utmx,utmy,izone,along,alat)
      else
          call mapcgl(rlat,rlon,utmx,utmy,alat,along)
      endif
      ALNMN=ALONG
      ALTMN=ALAT
      ALNMX=ALONG
      ALTMX=ALAT
      UTMX=UTMX+FLOAT(NXC*ICS)
      if(lutm)then
          call glat(utmx,utmy,izone,along,alat)
      else
          call mapcgl(rlat,rlon,utmx,utmy,alat,along)
      endif
      ALNMN=AMIN1(ALONG,ALNMN)
      ALTMN=AMIN1(ALAT,ALTMN)
      ALNMX=AMAX1(ALONG,ALNMX)
      ALTMX=AMAX1(ALAT,ALTMX)
C
      UTMY=UTMY+FLOAT(NYC*ICS)
      if(lutm)then
          call glat(utmx,utmy,izone,along,alat)
      else
          call mapcgl(rlat,rlon,utmx,utmy,alat,along)
      endif
      ALNMN=AMIN1(ALONG,ALNMN)
      ALTMN=AMIN1(ALAT,ALTMN)
      ALNMX=AMAX1(ALONG,ALNMX)
      ALTMX=AMAX1(ALAT,ALTMX)
C
      UTMX=FLOAT(IXO)
      if(lutm)then
          call glat(utmx,utmy,izone,along,alat)
      else
          call mapcgl(rlat,rlon,utmx,utmy,alat,along)
      endif
      ALNMN=AMIN1(ALONG,ALNMN)
      ALTMN=AMIN1(ALAT,ALTMN)
      ALNMX=AMAX1(ALONG,ALNMX)
      ALTMX=AMAX1(ALAT,ALTMX)
      LNMIN=INT(ALNMN)-1
      LNMAX=INT(ALNMX)+2
      LTMIN=INT(ALTMN)-1
      LTMAX=INT(ALTMX)+2
C
C
C     READ IN GIS DATA
C
      nread=0
      ntake=0
100   CONTINUE
      READ(7,1000,END=500) LAT1,LAT2,LON1,LON2,IPERC
      nread=nread+1
      IF (LAT1.LT.LTMIN) GO TO 100
      IF (LAT1.GT.LTMAX) GO TO 100
      IF (-LON1.LT.LNMIN) GO TO 100
      IF (-LON1.GT.LNMAX) GO TO 100
1000  FORMAT(4I4,11I5)
      ALAT1=FLOAT(LAT1)+FLOAT(LAT2)/12.0-DLAT
      ALONG1=-(FLOAT(LON1)+FLOAT(LON2)/8.0)-DLONG
      ALAT2=FLOAT(LAT1)+FLOAT(LAT2)/12.0+DLAT
      ALONG2=-(FLOAT(LON1)+FLOAT(LON2)/8.0)+DLONG
      if(lutm)then
          call gutm(along1,alat1,izone,utmx1,utmy1)
          call gutm(along2,alat1,izone,utmx2,utmy2)
          call gutm(along2,alat2,izone,utmx3,utmy3)
          call gutm(along1,alat2,izone,utmx4,utmy4)
      else
          CALL maplcg(rlat,rlon,alat1,along1,utmx1,utmy1)
          CALL maplcg(rlat,rlon,alat1,along2,utmx2,utmy2)
          CALL maplcg(rlat,rlon,alat2,along2,utmx3,utmy3)
          CALL maplcg(rlat,rlon,alat2,along1,utmx4,utmy4)
      endif
      UTXMN=AMIN1(UTMX1,UTMX2,UTMX3,UTMX4)
      UTYMN=AMIN1(UTMY1,UTMY2,UTMY3,UTMY4)
      UTXMX=AMAX1(UTMX1,UTMX2,UTMX3,UTMX4)
      UTYMX=AMAX1(UTMY1,UTMY2,UTMY3,UTMY4)
C
      I1=int((UTXMN-FLOAT(IXO))/FLOAT(ICS))-2
      I2=int((UTXMX-FLOAT(IXO))/FLOAT(ICS))+2
      J1=int((UTYMN-FLOAT(IYO))/FLOAT(ICS))-2
      J2=int((UTYMX-FLOAT(IYO))/FLOAT(ICS))+2
c
c...  skip data completely out of domain
      if(i1.lt.1 .and. i2.lt.1)then
          go to 100
      elseif(i1.gt.nxc .and. i2.gt.nxc)then
          go to 100
      elseif(j1.lt.1 .and. j2.lt.1)then
          go to 100
      elseif(j1.gt.nyc .and. j2.gt.nyc)then
          go to 100
      endif
      ntake=ntake+1
      I1=MAX0(I1,1)
      I2=MIN0(I2,NXC)
      J1=MAX0(J1,1)
      J2=MIN0(J2,NYC)
C
C*      WRITE(*,3301) I1,I2,J1,J2,LON1,LON2,LAT1,LAT2
C*3301  FORMAT(' WINDOW ',8I5)
      DO 200 J=J1,J2
        utmyc=yo+(float(j)-0.5)*float(ics)
      DO 200 I=I1,I2
        utmxc=xo+(float(i)-0.5)*float(ics)
c     WRITE(*,3302) I,J
3302  FORMAT(' FOUND ',2I5)
        if(lutm)then
            call glat(utmxc,utmyc,izone,alongc,alatc)
        else
            call mapcgl(rlat,rlon,utmxc,utmyc,alatc,alongc)
        endif
        if(alatc.lt.alat1 .or. alatc.gt.alat2)go to 200
        if(alongc.lt.along1 .or. alongc.gt.along2)go to 200
C
C     FOUND THIS GRID CELL
C
        PLAND(1,I,J)=pland(1,i,j)+FLOAT(IPERC(1))/100.0
        PLAND(2,I,J)=pland(2,i,j)+FLOAT(IPERC(2))/100.0
        PLAND(3,I,J)=pland(3,i,j)+FLOAT(IPERC(3))/100.0
        PLAND(4,I,J)=pland(4,i,j)+FLOAT(IPERC(4))/100.0
        PLAND(5,I,J)=pland(5,i,j)+FLOAT(IPERC(5))/100.0
        PLAND(6,I,J)=pland(6,i,j)+float(iperc(6))/100.0
        PLAND(7,I,J)=pland(7,i,j)+FLOAT(IPERC(7))/100.0
        PLAND(8,I,J)=pland(8,i,j)+FLOAT(IPERC(8))/100.0
        PLAND(9,I,J)=pland(9,i,j)+FLOAT(IPERC(9))/100.0
        PLAND(10,I,J)=pland(10,i,j)+float(iperc(10))/100.0
        PLAND(11,I,J)=pland(11,i,j)+float(iperc(11))/100.0
200   CONTINUE
      GO TO 100
500   CONTINUE
      print *,'Total number of landuse data records read: ',nread
      print *,'Total number of landuse data records chosen: ',ntake
c
c...  QA land use result
      nadj=0
      nzero=0
      nbig=0
      do 2901 i=1,nxc
      do 2901 j=1,nyc
        percmn=-99.9
        totper=0.0
        do 2902 k=1,11
          percck(i,j)=percck(i,j)+pland(k,i,j)
          if(pland(k,i,j) .gt. percmn)then
              percmn=pland(k,i,j)
              replu(i,j)=float(k)
          endif
          totper=totper+pland(k,i,j)
2902    continue
c
c...    Check data percentage coverage in the cell.  Print more info
c...    for step 2 adjustification if user desires (if not, statements
c...    can be commented out)
        if(totper .lt. 1.0e-4)then
            nzero=nzero+1
c           print *,'Cell I = ',i,' J = ',j,' has zero data coverage'
        elseif((totper-1.0) .lt. -1.0e-4)then
            nadj=nadj+1
c           print *,
c    *      'Cell I = ',i,' J = ',j,' has nonzero <100% data coverage: '
c    *              ,totper*100.0
c       elseif((totper-1.0) .gt. 1.0e-4)then
            nbig=nbig+1
c           print *,'Cell I = ',i,' J = ',j,' has >100% data coverage: '
c    *              ,totper*100.0
        endif
2901  continue
      print *,'Total number of cells with zero percentages: ',nzero
      print *,
     *'Total number of cells with nonzero <100% data coverage: ',nadj
      title='Landuse total percentages'
      call arry1d2d(dum1d,percck,2,nxc,nyc,mxx,mxy)
      call prnarry(title,0,0,nxc,nyc,dum1d,100.0)
      title='LU with max. percentages '
      call arry1d2d(dum1d,replu,2,nxc,nyc,mxx,mxy)
      call prnarry(title,0,0,nxc,nyc,dum1d,1.0)
C
C     CALCULATE TOTAL LAND USE TYPES FOR THIS REGION
C
      DO 290 K=1,11
         TOT(K)=0.0
290   CONTINUE
      DO 300 J=1,NYC
      DO 300 I=1,NXC
C
        DO 310 K=1,11
           TOT(K)=TOT(K)+PLAND(K,I,J)/FLOAT(NXC*NYC)
310     CONTINUE
300   CONTINUE
      WRITE(12,2800)
2800  FORMAT(//,5X,'PERCENT COVERAGE OF LAND USE CLASSES IN MESOSCALE'
     1,' MODELING DOMAIN',//,5X,'LAND USE CLASS',11X,'PERCENT',/)
      DO 299 K=1,11
         PP=100.0*TOT(K)
         WRITE(12,2900) NAMLND(K),PP
299   CONTINUE
2900  FORMAT(5X,A20,F10.3)
c
c...  Write out land use data
c
3011    format(2i11,2i5,a4,f6.2)
      do 302 j=1,nyc
      do 302 i=1,nxc
        write(8,3021)i,j,(pland(k,i,j),k=1,11)
3021    format(i4,i4,11f10.3)
302   continue
      STOP
      END
c----------------------------------------------------------------
      SUBROUTINE GLAT(UTMX,UTMY,IZONE,ALON,ALAT)
c
c --- PRELND1 Version: 3.0           Level: 941215           GLAT
c----------------------------------------------------------------
CDECK.GLAT
C
C  DATE: NOVEMBER 1987
C  VERSION: ARM3-1.0
C
C
C     CONVERT UTM COORDIANTES TO LAT/LONG
C     NOTE: LONG WILL BE NEGATIVE
C  INPUT ARGUMENTS:
C                   UTMX   R  UTM COORDINATE IN THE X-DIRECTION (KM)
C                   UTMY   R  UTM COORDINATE IN THE Y-DIRECTION (KM)
C                   IZONE  I  UTM ZONE
C
C  OUTPUT ARGUMENTS:
C                   ALONG  R  LONGITUDE OF COORDINATE (NEGATIVE IN USA)
C                   ALAT   R  LATITUDE OF COORDINATE (DEGREES)
C
C  SUBROUTINES CALLED:
C                   MAPUTG
C
C  CALLED BY:
C                   PRELND
C
C
C
      CALL MAPUTG(UTMX,UTMY,IZONE,ALON,ALAT)
C*    ALON=-ALON
C*    LAT1=INT(ALAT)
C*    LAT2=INT((ALAT-FLOAT(LAT1))*60.)
C*    LAT3=INT((ALAT-FLOAT(LAT1)-FLOAT(LAT2)/60.)*3600.0)
C*    LON1=INT(ALON)
C*    LON2=INT((ALON-FLOAT(LON1))*60.)
C*    LON3=INT((ALON-FLOAT(LON1)-FLOAT(LON2)/60.)*3600.0)
C*    WRITE(1,1200) LAT1,LAT2,LAT3,LON1,LON2,LON3
C*00  FORMAT(' LATITUDE= ',3I5,10X,'LONGITUDE= ',3I5)
C*    GO TO 10
      RETURN
      END
c----------------------------------------------------------------
      SUBROUTINE GUTM(ALONG,ALAT,IZONE,UTMX,UTMY)
c
c --- PRELND1 Version: 3.0           Level: 941215           GUTM
c----------------------------------------------------------------
C
CDECK.GUTM
C
C  DATE: NOVEMBER 1987
C  VERSION: ARM3-1.0
C
C     CONVERT LONG/LAT TO UTM COORDIANTES
C
C  INPUT ARGUMENTS:
C                   ALONG   R  LONGITUDE OF COORDINATE (DEGREES)
C                   ALAT   R  LATITUDE OF COORDINATE (DEGREES)
C                   IZONE  I  ZONE UTM COORDIANTES ARE REQUIRED IN
C
C  OUTPUT ARGUMENTS:
C                   UTMX   R  UTM COORDINATE IN X DIRECTION (KM) IZONE
C                   UTMY   R  UTM COORDINATE IN Y DIRECTION (KM) IZONE
C
C  SUBROUTINES CALLED:
C                   MAPGTU
C
C  CALLED BY:
C                   PRELND
C
C
C****************************************************************
C
C
C     WIRED FOR USA
C
      ALON=ABS(ALONG)
      IZ=IZONE
      CALL MAPGTU (-ALON,ALAT,IZ,UTMX,UTMY)
      RETURN
      END
C
      INTEGER FUNCTION INSIDE (X,Y,NPTS,PTS)
C
C     *** DETERMINES IF A POINT IS INSIDE A POLYGON
C             GW LUNDBERG/SAI  APRIL '83
C
C         X,Y   THE POINT IN DATA UNITS
C         NPTS  THE NUMBER OF POINTS IN THE POLYGON
C         PTS   THE ARRAY OF POLYGON POINTS (X,Y) IN DATA SPACE
C
C     *** NOTE -- TO INCLUDE BOUNDARY, SET INSIDE = 1
C                 AT $200, TO EXCLUDE BOUNDARY, SET
C                 INSIDE = 0 AT $200.
C
      DIMENSION PTS(2,*)
C
C     ---------------------------------------------------------
C
C     RESET CROSSINGS COUNT
      KNT = 0
C
C     *** LOOP OVER ALL LINE SEGMENTS OF POLYGON
      DO 100 I=1,NPTS
         IP1 = I+1
C
C        *** ADJUST FOR WRAP AROUND
         IF (I .EQ. NPTS) IP1 = 1
C
C        *** TRANSLATE SO THAT (X,Y) IS THE ORIGIN
         X1 = PTS(1,I) - X
         Y1 = PTS(2,I) - Y
C
C        *** CHECK IF (X,Y) ON THIS NODE POINT
         IF (X1 .EQ. 0.  .AND.  Y1 .EQ. 0.) GO TO 200
C
         X2 = PTS(1,IP1) - X
         Y2 = PTS(2,IP1) - Y
C
C        *** DIRECT SEGMENT SO THAT X2>=X1
         IF (X2 .GE. X1) GO TO 20
         T = X1
         X1 = X2
         X2 = T
         T = Y1
         Y1 = Y2
         Y2 = T
C
C        *** IF HIGH X BELOW Y-AXIS -- IGNORE THIS SEGMENT
   20    IF (X2 .LT. 0.) GO TO 100
C
C        *** IF LOW X BELOW Y-AXIS -- PUNT
         IF (X1 .LT. 0.) GO TO 30
C
C        *** CAN CROSS POSITIVE X-RAY ONLY IF SIGNS DIFFER
         IF (SIGN(1.,Y1) .EQ. SIGN(1.,Y2)) GO TO 100
C
C        *** CROSSES -- CHECK ON BOUNDARY
         IF (X1 .EQ. 0.  .AND.  X2 .EQ. 0.) GO TO 200
C
C        *** VALID CROSSING
         GO TO 90
C
C
C        *** REDETERMINE Y1 AT Y-AXIS AND TEST AGAIN
   30    CONTINUE
         Y1 = -X1*(Y2-Y1)/(X2-X1) + Y1
C
C        *** CHECK ON BOUNDARY
   40    IF (Y1 .EQ. 0.) GO TO 200
C
C        *** CHECK ON CROSSING
         IF (SIGN(1.,Y1) .EQ. SIGN(1.,Y2)) GO TO 100
C
C        *** CROSSES -- BUMP CROSSING COUNT
   90    KNT = KNT+1
  100 CONTINUE
C
C     *** POINT INSIDE IF COUNT ODD
C**
C**INSIDE = RT (KNT,1)
C**
C** MODIFIED 11/87 SAI/REM TO ASSURE COMPUTER TRANSPORTABILITY (ARM3-1.0)
      INSIDE=MOD(KNT,2)
      RETURN
C
C     *** ON BOUNDARY -- HENCE -- INSIDE
  200 INSIDE = 1
      RETURN
      END
c----------------------------------------------------------------
      Subroutine mapcgl(rlat,rlon,xdist,ydist,alat,alon)
c
c --- PRELND1 Version: 3.0           Level: 941215        MAPCGL
C----------------------------------------------------------------
C
C-----This is a specialized routine to use the Lambert Conformal
C-----projection, rather than UTM's as an approximate rectilinear
C-----grid.
C
C-----The inputs are the following:
C
C-----rlon = longitude of domain center (i.e. coordinate reference
c...         point); in degrees; < 0 in the USA
c...  rlat = latitude of domain center (deg.)
c-----xdist = x distance from origin (km)
C-----ydist = y distance from origin (km)
C
C-----The outputs are
C-----longitude (deg)
C-----latitude (deg)
C
C
C-----This program was written and tested by G Moore July 17, 1992
C-----This is version 1 Sigma Research Corp (a256)
c
c...  Modified by Ed Chang; SRC; 8/93; A314
c...     (1) Hardwire TLAT1 = 30 degrees N.; TLAT2 = 60 degrees N.
c...     (2) Eliminate XOFF, YOFF:  Offsets from domain origin.
c...     (3) Add variable ALAT
c...     (4) Add common block CSTLC so that mapping constants are
c...         calculated only the first time through.
C
C**********************************************************************
C
C
C-----Estimates and sets some intial constants
C

      common /cstlc/pi,pi4,radcon,rearth,psi,rho1,cc,a1,a2,a3,a5
c
      data tlat1,tlat2/30.0, 60.0/
c
      if(abs(psi) .lt. 1.0e-20)then
          pi = acos(-1.0)
          pi4 = pi/4.0
          radcon = 2.0*pi/360.
          rearth = 6370.0
          a1 = cos(tlat1*radcon)/cos(tlat2*radcon)
          a2 = tan(pi4 - (radcon*tlat1)/2.0)
          a3 = tan(pi4 - (radcon*tlat2)/2.0)
          a5 = tan(pi4 - (radcon*rlat)/2.0)
C-----    Compute the cone constant (cc - K in other people's notation)
C
          cc = alog(a1)/alog(a2/a3)
C-----    Scaling function
          psi = rearth*cos(tlat1*radcon)/(cc*a2**cc)
          rho1 = psi*a5**cc
      endif
C
C-----Start estimate of (x,y) to (alat,alon)
C
C-----longitude estimate
      beta = (rho1 - ydist)
      theta = atan(xdist/beta)
      alon = rlon + theta/(cc*radcon)
C-----latitude estimate
      sin1 = psi*cos(theta)
      c1 = 1.0/cc
c     c2 = (xdist/sin(theta))**c1
      c2 = beta**c1
      c3 = sin1**c1
      pi2=pi/2.0
      alat = (pi2-2.0*atan2(c2,c3))/radcon
c     alat = (2.0*(pi4 - atan(c2)))/radcon
      Return
      End
CDECK MAPGTU
c----------------------------------------------------------------
      SUBROUTINE MAPGTU (ALON,ALAT,IZONE,UTMX,UTMY)
c
c --- PRELND1 Version: 3.0           Level: 941215         MAPGTU
c----------------------------------------------------------------
C
C     *** CONVERTS LATITUDE AND LONGITUDE TO UTM COORDINATES
C         THIS IS A MODIFICATION OF THE US GEOLOGICAL SURVEY
C         PROGRAM NO J380 (TOPOGRAPHIC DIVISION)
C             GW LUNDBERG/SAI  APR 79
C
C         ALON    LONGITUDE IN DEGREES (WESTERN HEMISPHERE IS NEGATIVE)
C         ALAT    LATITUDE IN DEGREES
C         IZONE   ZONE OVERRIDE IF NOT ZERO
C         UTMX    EASTING (KM)
C         UTMY    NORTHING (KM)
C
      DIMENSION B(12)
      COMMON /VVUTM/ A(16)
C
      SLAT = ALAT*3600.
      SLON = -ALON*3600.
C
C     *** COMPUTE COEFFICIENTS FOR CONVERSION
      CALL VVGTU1
C
C     *** TEST FOR ZONE INPUT ON GEODETIC TO UTM INDICATING OVERRIDE OF
C         NORMAL 6 DEGREE LONGITUDE BAND.
C
      IF(IZONE.EQ.0)GO TO 30
C
C     *** COMPUTE CENTRAL MERIDIAM IN SECONDS FOR ENFORCED ZONE.
      IF(IABS(IZONE)-30)10,10,20
   10 UTZ=30.0-IABS(IZONE)
      A(9)=((UTZ*6.0)+3.0)*3600.0
      GO TO 40
   20 UTZ=IABS(IZONE)-30.0
      A(9)=((UTZ*6.0)-3.0)*(-3600.0)
      GO TO 40
C
C     *** COMPUTE UTM ZONE(IZONE) AND CENTRAL MERIDIAN IN SECONDS(A9)
C         FOR GEODETIC TO UTM CONVERSION WHERE ZONE IS NOT INPUT.
C
   30 IZONE=30-(INT(SLON)/21600)
      UTZ=30.0-IZONE
      A(9)=((UTZ*6.0)+3.0)*3600.0
C
C     *** CONVERT GEODETIC TO UTM COORDINATES
C
C     *** RETURNS ZEROS IF LATITUDE EXCEEDS 84 DEGREES OR LONGITUDE
C         0.16 RADIANS
   40 IF(ABS(SLAT)-302400.0) 80,80,70
   70 X=0.0
      Y=0.0
      GO TO 99
   80 B(10)=(A(9)-SLON) *4.848181109536E-6
      IF(ABS(B(10))-0.16)90,90,70
   90 B(9)=SLAT*4.848136811095E-6
      SINP=SIN(B(9))
      COSP=COS(B(9))
      RN=A(15)/SQRT(1.0-A(16)*SINP*SINP)
      T=SINP/COSP
      TS=T*T
      B(11)=COSP*COSP
      ETAS=A(16)*B(11)/(1.0-A(16))
      B(1)=RN*COSP
      B(3)=(1.0-TS+ETAS)*B(1)*B(11)/6.0
      B(5)=((TS-18.0)*TS+5.0+(14.0-58.0*TS)*ETAS)*B(1)*B(11)*
     $ B(11)/120.0
      B(7)=(((179.0-TS)*TS-479.0)*TS+61.0)*B(1)*B(11)**3/5040.0
      B(12)=B(10)*B(10)
      X=(((B(7)*B(12)+B(5))*B(12)+B(3))*B(12)+B(1))*B(10)*A(8)+
     $  A(5)
      B(2)=RN*B(11)*T/2.0
      B(4)=(ETAS*(9.0+4.0*ETAS)+5.0-TS)*B(2)*B(11)/12.0
      B(6)=((TS-58.0)*TS+61.0+(270.0-330.0*TS)*ETAS)*B(2)*
     $ B(11)*B(11)/360.0
      B(8)=(((543.0-TS)*TS-3111.0)*TS+1385.0)*B(2)*B(11)**3/
     $ 20160.0
      Y=(((B(8)*B(12)+B(6))*B(12)+B(4))*B(12)+B(2))*B(12)+
     $ ((((A(4)*B(11)+A(3))*B(11)+A(2))*B(11)+A(1))*SINP*COSP+B(9))
     $ *A(10)
      Y=(Y-A(7))*A(8)+A(6)
C
   99 UTMX = X/1000.
      UTMY = Y/1000.
      RETURN
      END
c----------------------------------------------------------------
      Subroutine maplcg(rlat,rlon,alat,alon,xdist,ydist)
c
c --- PRELND1 Version: 3.0           Level: 941215        MAPLCG
c----------------------------------------------------------------
C
C-----This is a specialized routine to use the Lambert Conformal
C-----projection, rather than UTM's as an approximate rectilinear
C-----grid.
C
C-----The inputs are the following:
C
C-----rlon = longitude of domain center (i.e. coordinate reference
c...         point); in degrees;  < 0 for USA
C...  rlat = latitude (deg) of domain center
C
C-----The outputs are
C
C-----xdist = x distance from origin (km)
C-----ydist = y distance from origin (km)
C
C-----This program was written and tested by G Moore July 17, 1992
C-----This is version 1 Sigma Research Corp (a256)
c
c...  Modified by Ed Chang, SRC; 8/93; A314
c...     (1) Hardwire TLAT1=30 degrees; TLAT2 = 60 degrees
c...     (2) Eliminate XOFF; YOFF in the original program
c...         XOFF;YOFF:  offset from grid origin
c...     (3) Add variable RLAT
c...     (4) Add common block CSTLC so that mapping constants are
c...         calculated only the first time through.
C
C**********************************************************************
C
C
C-----Estimates and sets some intial constants
C

      common /cstlc/pi,pi4,radcon,rearth,psi,rho1,cc,a1,a2,a3,a5
c
      data tlat1,tlat2/30.0,60.0/
c
      if(abs(psi) .lt. 1.0e-20)then
          pi = acos(-1.0)
          pi4 = pi/4.0
          radcon = 2.0*pi/360.
          rearth = 6370.0
          a1 = cos(tlat1*radcon)/cos(tlat2*radcon)
          a2 = tan(pi4 - (radcon*tlat1)/2.0)
          a3 = tan(pi4 - (radcon*tlat2)/2.0)
          a5 = tan(pi4 - (radcon*rlat)/2.0)
C-----    Compute the cone constant (cc - K in other people's notation)
C
          cc = alog(a1)/alog(a2/a3)
C-----    Scaling function
          psi = rearth*cos(tlat1*radcon)/(cc*a2**cc)
          rho1 = psi*a5**cc
      endif
      a4 = tan(pi4 - (radcon*alat)/2.0)
C
C-----Start estimate of (alat,alon) to (x,y)
C-----Polar angle
      theta = (alon - rlon)*cc*radcon
C-----polar radiuses (origin,latitude(alat))
      rho = psi*a4**cc
C-----Cartesian distances for a round earth in meters (S=1)
      xdist = rho*sin(theta)
      ydist = rho1 - rho*cos(theta)
      Return
      End
CDECK MAPUTG *********************************************************
c----------------------------------------------------------------
      SUBROUTINE MAPUTG (UTMX,UTMY,IZONE,ALON,ALAT)
c
c --- PRELND1 Version: 3.0           Level: 941215         MAPUTG
c----------------------------------------------------------------
C
C     *** CONVERTS UTM COORDINATES TO LATITUDE AND LONGITUDE
C         THIS IS A MODIFICATION OF THE US GEOLOGICAL SURVEY
C         PROGRAM NO J380  (TOPOGRAPHIC DIVISION)
C             GW LUNDBERG/SAI  APR 79
C
C         UTMX    EASTING IN KILOMETERS
C         UTMY    NORTHING IN KILOMETERS
C         IZONE   UTM ZONE NUMBER
C         ALON    LONGITUDE IN DEGREES
C         ALAT    LATITUDE IN DEGREES
C
      DIMENSION B(12)
      COMMON /VVUTM/ A(16)
C
      X = UTMX*1000.
      Y = UTMY*1000.
C
C     *** COMPUTE CONVERSION COEFFICIENTS IF NEEDED
      CALL VVGTU1
C
C     *** COMPUTE CENTRAL MERIDIAN IN SECONDS FROM IZONE INPUT
   10 UTZ = 30.0 - IABS (IZONE)
      A(9) = ((UTZ*6.0) + 3.0) * 3600.0
C
C     *** CONVERT UTM COORDINATES TO GEODETIC
      B(9)=((A(5)-X)*1.0E-6)/A(8)
      IF (ABS(B(9))-1.0E-7*A(15)*2.0)30,30,20
   20 SLAT=0.0
      SLON=0.0
      GO TO 99
   30 B(10)=((Y-A(6))/A(8)+A(7))/A(10)
      IF(ABS(B(10))-1.47E0)40,40,20
   40 SINW=SIN(B(10))
      COSW=COS(B(10))
      B(12)=COSW*COSW
      B(11)=(((A(14)*B(12)+A(13))*B(12)+A(12))*B(12)+A(11))*SINW*COSW
     $ +B(10)
      SINW=SIN(B(11))
      COSW=COS(B(11))
      RN=SQRT(1.0-A(16)*SINW*SINW)*1.0E6/A(15)
      T=SINW/COSW
      TS=T*T
      B(12)=COSW*COSW
      ETAS=A(16)*B(12)/(1.0-A(16))
      B(1)=RN/COSW
      B(2)=-T*(1.0+ETAS)*RN*RN/2.0
      B(3)=-(1.0+2.0*TS+ETAS)*B(1)*RN*RN/6.0
      B(4)=(((-6.0-ETAS*9.0)*ETAS+3.0)*TS+(6.0-ETAS*3.0)*ETAS
     $ +5.0)*T*RN**4/24.0
      B(5)=((TS*24.0+ETAS*8.0+28.0)*TS+ETAS*6.0+5.0)*B(1)*
     $ RN**4/120.0
      B(6)=(((ETAS*45.0-45.0)*TS+ETAS*162.0-90.0)*TS
     $ -ETAS*107.0-61.0)*T*RN**6/720.0
      B(7)=-(((TS*720.0+1320.0)*TS+662.0)*TS+61.0)*B(1)*RN**6/
     $ 5040.0
      B(8)=(((TS*1575.0+4095.0)*TS+3633.0)*TS+1385.0)*T*RN**8/
     $ 40320.0
      B(10)=B(9)*B(9)
      SLAT=((((B(8)*B(10)+B(6))*B(10)+B(4))*B(10)+B(2))*B(10)+B(11))*
     $ 206264.80624709
      SLON= (((B(7)*B(10)+B(5))*B(10)+B(3))*B(10)+B(1))*B(9)*
     $ 206264.80624709 + A(9)
C
C     *** CONVERT SECONDS OF ARC TO DEGREES AND RETURN
   99 ALAT = SLAT/3600.
      ALON = -SLON/3600.
C
      RETURN
      END
c----------------------------------------------------------------
        subroutine prnarry(title,itime,ndate,nx,ny,xout1d,factor)
c
c --- PRELND1 Version: 3.0           Level: 941215        PRNARRY
c----------------------------------------------------------------
c
c
c...    This subroutine prints out contents of an array
c...                                    ----- CCC, October, 1991, SAI
c
        dimension xout1d(nx*ny),iz(30),xout(104,100)
        character*25 title
c
        npan=nint(real(nx)/30.0+0.5)
        call arry1d2d(xout1d,xout,1,nx,ny,104,100)
        do 1 ipan=1,npan
          i1=1+(ipan-1)*30
          if(i1 .gt. nx)go to 1
          i2=i1-1+30
          if(i2 .gt. nx)i2=nx
          write(12,100)title,itime,ndate,1.0/factor,ipan,(i,i=i1,i2)
100       format(1h1,5x,a25,5x,'[TIME] = ',i4,'  [DATE] = ',i6,
     *           ' [SCALE]= x',1pe10.3,' [PANEL]=',i1,
     *           /,6x,30(1x,i2,1x),/,6x,30(' ---'))
          mxxprn=30
          if(ipan .eq. 1)then
              if(i2 .lt. 30)mxxprn=i2
          elseif(ipan .eq. 2)then
              if(i2 .lt. 60) mxxprn=i2-30
          else
              if(i2 .lt. 90)mxxprn=i2-60
          endif
          do 12 jj=1,ny
            j=ny-jj+1
            ii=0
            do 111 i=i1,i2
              ii=i-30*(ipan-1)
              iz(ii)=nint(xout(i,j)*factor)
111         continue
            write(12,200)j,(iz(i),i=1,mxxprn)
200         format(1x,i2,' : ',30i4)
12        continue
1       continue
        return
        end
c----------------------------------------------------------------
        subroutine arry1d2d(A,B,ntype,kkx,kky,mxkx,mxky)
c
c --- PRELND1 Version: 3.0           Level: 941215       ARRY1D2D
c----------------------------------------------------------------
c
c
c...    This program transforms an 1D array (A) to a 2D array (B) or vice
c...    versa
c
c
c...    'ntype' = 1:  1D to 2D
c...    'ntype' = 2:  2D to 1D
c
        dimension a(kkx*kky),b(mxkx,mxky)
c
c       if(mxkx*mxky .ne. mxkxy)stop 'ARRY1D2D'
        nn=0
        if(ntype .eq. 1)then
            do 1 i=1,kkx
            do 1 j=1,kky
              nn=nn+1
              b(i,j)=a(nn)
1           continue
        elseif(ntype .eq. 2)then
            do 2 i=1,kkx
            do 2 j=1,kky
              nn=nn+1
              a(nn)=b(i,j)
2           continue
        else
            print *,'Error: Wrong type of array transfer, see ARRY1D2D'
            stop
        endif
        return
        end
CDECK VVGTU1 ***********************************************************
c----------------------------------------------------------------
      SUBROUTINE VVGTU1
c
c --- PRELND1 Version: 3.0           Level: 941215         VVGTU1
c----------------------------------------------------------------
C
C     *** SETS UP COEFFICIENTS FOR CONVERTING GEODETIC TO
C         RECTIFYING LATITUDE AND CONVERSELY
C
C         A1-A4 COEFFICIENTS FOR CONVERTING GEODITIC TO RECTIFYING LATITUDE
C         A5    FALSE EASTING
C         A6    FALSE NORTHING
C         A8    SCALE FACTOR AT CENTRAL MERIDIAN
C         A9    CENTRAL MERIDIAN IN SECONDS
C         A10   RADIUS OF SPHERE HAVING GREAT CIRCLE LENGTH EQUAL TO
C               SPHEROID MERIDIAN LENGTH
C         A11-A14  COEFFICIENTS FOR CONVERTING RECTIFYING LATITUDE TO
C                  GEODETIC LATITUDE
C         A15   SEMIMAJOR AXIS OF SPHERIOD
C         A16   ECCENTRICITY SQUARED
C
C
      COMMON /VVUTM/ A(16)
      DATA IFRST/1/
C
C     *** CONPUTE COEFFICIENTS ON FIRST PASS ONLY
      IF (IFRST .NE. 1) RETURN
      IFRST = 0
C
      A(5)=5.0E5
      A(6)=0.0
      A(7)=0.0
      A(8)=0.9996
C
C     *** CLARKE 1866 ELLIPSOID AXES LENGTHS
      A(15)=6378206.4
      B = 6356583.80
C
      A(16)= ((A(15)-B)/A(15))*((A(15)+B)/A(15))
      A(10)=(((A(16)*(7.0/3.2E1)+(5.0/1.6E1))*A(16)+0.5)*A(16)
     $ +1.0)*A(16)*0.25
      A(1)=-(((A(10)*(1.95E2/6.4E1)+3.25)*A(10)+3.75)*A(10)+3.0)*
     $ A(10)
      A(2)=(((1.455E3/3.2E1)*A(10)+(7.0E1/3.0))*A(10)+7.5)*A(10)**2
      A(3)=-((7.0E1/3.0)+A(10)*(9.45E2/8.0))*A(10)**3
      A(4)=(3.15E2/4.0)*A(10)**4
      A(11)=(((7.75-(6.57E2/6.4E1)*A(10))*A(10)-5.25)*A(10)+3.0)*
     $ A(10)
      A(12)=(((5.045E3/3.2E1)*A(10)-(1.51E2/3.0))*A(10)+10.5)*
     $ A(10)**2
      A(13)=((1.51E2/3.0)-(3.291E3/8.0)*A(10))*A(10)**3
      A(14)=(1.097E3/4.0)*A(10)**4
      FAC=A(10)*A(10)
      A(10)=(((2.25E2/6.4E1)*FAC+2.25)*FAC+1.0)*(1.0-FAC)*
     $ (1.0-A(10))*A(15)
      RETURN
      END
